@@ -1,36 +1,31 @@
 # -*- coding: utf-8 -*-
-"""
-Python client for InfluxDB
-"""
+"""Python client for InfluxDB."""
+
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 from __future__ import unicode_literals
 
+import time
+import random
+
 import json
 import socket
 import requests
 import requests.exceptions
-from sys import version_info
+from six.moves import xrange
+from six.moves.urllib.parse import urlparse
 
 from influxdb.line_protocol import make_lines, quote_ident, quote_literal
 from influxdb.resultset import ResultSet
 from .exceptions import InfluxDBClientError
 from .exceptions import InfluxDBServerError
 
-try:
-    xrange
-except NameError:
-    xrange = range
-
-if version_info[0] == 3:
-    from urllib.parse import urlparse
-else:
-    from urlparse import urlparse
-
 
 class InfluxDBClient(object):
-    """The :class:`~.InfluxDBClient` object holds information necessary to
+    """InfluxDBClient primary client object to connect InfluxDB.
+
+    The :class:`~.InfluxDBClient` object holds information necessary to
     connect to InfluxDB. Requests can be made to InfluxDB directly through
     the client.
 
@@ -42,6 +37,8 @@ class InfluxDBClient(object):
     :type username: str
     :param password: password of the user, defaults to 'root'
     :type password: str
+    :param pool_size: urllib3 connection pool size, defaults to 10.
+    :type pool_size: int
     :param database: database name to connect to, defaults to None
     :type database: str
     :param ssl: use https instead of http to connect to InfluxDB, defaults to
@@ -62,6 +59,8 @@ class InfluxDBClient(object):
     :type udp_port: int
     :param proxies: HTTP(S) proxy to use for Requests, defaults to {}
     :type proxies: dict
+    :param path: path of InfluxDB on the server to connect, defaults to ''
+    :type path: str
     """
 
     def __init__(self,
@@ -77,6 +76,8 @@ class InfluxDBClient(object):
                  use_udp=False,
                  udp_port=4444,
                  proxies=None,
+                 pool_size=10,
+                 path='',
                  ):
         """Construct a new InfluxDBClient object."""
         self.__host = host
@@ -89,56 +90,76 @@ class InfluxDBClient(object):
 
         self._verify_ssl = verify_ssl
 
-        self.use_udp = use_udp
-        self.udp_port = udp_port
+        self.__use_udp = use_udp
+        self.__udp_port = udp_port
         self._session = requests.Session()
+        adapter = requests.adapters.HTTPAdapter(
+            pool_connections=int(pool_size),
+            pool_maxsize=int(pool_size)
+        )
+
         if use_udp:
             self.udp_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
 
+        if not path:
+            self.__path = ''
+        elif path[0] == '/':
+            self.__path = path
+        else:
+            self.__path = '/' + path
+
         self._scheme = "http"
 
         if ssl is True:
             self._scheme = "https"
 
+        self._session.mount(self._scheme + '://', adapter)
+
         if proxies is None:
             self._proxies = {}
         else:
             self._proxies = proxies
 
-        self.__baseurl = "{0}://{1}:{2}".format(
+        self.__baseurl = "{0}://{1}:{2}{3}".format(
             self._scheme,
             self._host,
-            self._port)
+            self._port,
+            self._path)
 
         self._headers = {
-            'Content-type': 'application/json',
+            'Content-Type': 'application/json',
             'Accept': 'text/plain'
         }
 
     @property
     def _baseurl(self):
-        return self._get_baseurl()
-
-    def _get_baseurl(self):
         return self.__baseurl
 
     @property
     def _host(self):
-        return self._get_host()
-
-    def _get_host(self):
         return self.__host
 
     @property
     def _port(self):
-        return self._get_port()
-
-    def _get_port(self):
         return self.__port
 
+    @property
+    def _path(self):
+        return self.__path
+
+    @property
+    def _udp_port(self):
+        return self.__udp_port
+
+    @property
+    def _use_udp(self):
+        return self.__use_udp
+
     @classmethod
-    def from_DSN(cls, dsn, **kwargs):
-        """Return an instance of :class:`~.InfluxDBClient` from the provided
+    def from_dsn(cls, dsn, **kwargs):
+        r"""Generate an instance of InfluxDBClient from given data source name.
+
+        Return an instance of :class:`~.InfluxDBClient` from the provided
         data source name. Supported schemes are "influxdb", "https+influxdb"
         and "udp+influxdb". Parameters for the :class:`~.InfluxDBClient`
         constructor may also be passed to this method.
@@ -153,12 +174,12 @@ class InfluxDBClient(object):
 
         ::
 
-            >> cli = InfluxDBClient.from_DSN('influxdb://username:password@\
-localhost:8086/databasename', timeout=5)
+            >> cli = InfluxDBClient.from_dsn('influxdb://username:password@\
+            localhost:8086/databasename', timeout=5)
             >> type(cli)
             <class 'influxdb.client.InfluxDBClient'>
-            >> cli = InfluxDBClient.from_DSN('udp+influxdb://username:pass@\
-localhost:8086/databasename', timeout=5, udp_port=159)
+            >> cli = InfluxDBClient.from_dsn('udp+influxdb://username:pass@\
+            localhost:8086/databasename', timeout=5, udp_port=159)
             >> print('{0._baseurl} - {0.use_udp} {0.udp_port}'.format(cli))
             http://localhost:8086 - True 159
 
@@ -167,8 +188,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
             be used for the TCP connection; specify the UDP port with the
             additional `udp_port` parameter (cf. examples).
         """
-
-        init_args = parse_dsn(dsn)
+        init_args = _parse_dsn(dsn)
         host, port = init_args.pop('hosts')[0]
         init_args['host'] = host
         init_args['port'] = port
@@ -247,14 +267,17 @@ localhost:8086/databasename', timeout=5, udp_port=159)
                     timeout=self._timeout
                 )
                 break
-            except requests.exceptions.ConnectionError as e:
+            except (requests.exceptions.ConnectionError,
+                    requests.exceptions.HTTPError,
+                    requests.exceptions.Timeout):
                 _try += 1
                 if self._retries != 0:
                     retry = _try < self._retries
-
-        else:
-            raise requests.exceptions.ConnectionError
-
+                if method == "POST":
+                    time.sleep((2 ** _try) * random.random() / 100.0)
+                if not retry:
+                    raise
+        # if there's not an error, there must have been a successful response
         if 500 <= response.status_code < 600:
             raise InfluxDBServerError(response.content)
         elif response.status_code == expected_response_code:
@@ -269,6 +292,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :param data: the data to be written
         :type data: (if protocol is 'json') dict
                     (if protocol is 'line') sequence of line protocol strings
+                                            or single string
         :param params: additional parameters for the request, defaults to None
         :type params: dict
         :param expected_response_code: the expected response code of the write
@@ -279,9 +303,8 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :returns: True, if the write operation is successful
         :rtype: bool
         """
-
         headers = self._headers
-        headers['Content-type'] = 'application/octet-stream'
+        headers['Content-Type'] = 'application/octet-stream'
 
         if params:
             precision = params.get('precision')
@@ -291,6 +314,8 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         if protocol == 'json':
             data = make_lines(data, precision).encode('utf-8')
         elif protocol == 'line':
+            if isinstance(data, str):
+                data = [data]
             data = ('\n'.join(data) + '\n').encode('utf-8')
 
         self.request(
@@ -303,7 +328,8 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         )
         return True
 
-    def _read_chunked_response(self, response, raise_errors=True):
+    @staticmethod
+    def _read_chunked_response(response, raise_errors=True):
         result_set = {}
         for line in response.iter_lines():
             if isinstance(line, bytes):
@@ -311,8 +337,9 @@ localhost:8086/databasename', timeout=5, udp_port=159)
             data = json.loads(line)
             for result in data.get('results', []):
                 for _key in result:
-                    if type(result[_key]) == list:
-                        result_set.setdefault(_key, []).extend(result[_key])
+                    if isinstance(result[_key], list):
+                        result_set.setdefault(
+                            _key, []).extend(result[_key])
         return ResultSet(result_set, raise_errors=raise_errors)
 
     def query(self,
@@ -323,13 +350,15 @@ localhost:8086/databasename', timeout=5, udp_port=159)
               database=None,
               raise_errors=True,
               chunked=False,
-              chunk_size=0):
+              chunk_size=0,
+              method="GET"):
         """Send a query to InfluxDB.
 
         :param query: the actual query string
         :type query: str
 
-        :param params: additional parameters for the request, defaults to {}
+        :param params: additional parameters for the request,
+            defaults to {}
         :type params: dict
 
         :param epoch: response timestamps to be in epoch format either 'h',
@@ -356,6 +385,9 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :param chunk_size: Size of each chunk to tell InfluxDB to use.
         :type chunk_size: int
 
+        :param method: the HTTP method for the request, defaults to GET
+        :type method: str
+
         :returns: the queried data
         :rtype: :class:`~.ResultSet`
         """
@@ -373,9 +405,12 @@ localhost:8086/databasename', timeout=5, udp_port=159)
             if chunk_size > 0:
                 params['chunk_size'] = chunk_size
 
+        if query.lower().startswith("select ") and " into " in query.lower():
+            method = "POST"
+
         response = self.request(
             url="query",
-            method='GET',
+            method=method,
             params=params,
             data=None,
             expected_response_code=expected_response_code
@@ -395,8 +430,8 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         # TODO(aviau): Always return a list. (This would be a breaking change)
         if len(results) == 1:
             return results[0]
-        else:
-            return results
+
+        return results
 
     def write_points(self,
                      points,
@@ -439,7 +474,6 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         .. note:: if no retention policy is specified, the default retention
             policy for the database is used
         """
-
         if batch_size and batch_size > 0:
             for batch in self._batches(points, batch_size):
                 self._write_points(points=batch,
@@ -448,14 +482,28 @@ localhost:8086/databasename', timeout=5, udp_port=159)
                                    retention_policy=retention_policy,
                                    tags=tags, protocol=protocol)
             return True
-        else:
-            return self._write_points(points=points,
-                                      time_precision=time_precision,
-                                      database=database,
-                                      retention_policy=retention_policy,
-                                      tags=tags, protocol=protocol)
 
-    def _batches(self, iterable, size):
+        return self._write_points(points=points,
+                                  time_precision=time_precision,
+                                  database=database,
+                                  retention_policy=retention_policy,
+                                  tags=tags, protocol=protocol)
+
+    def ping(self):
+        """Check connectivity to InfluxDB.
+
+        :returns: The version of the InfluxDB the client is connected to
+        """
+        response = self.request(
+            url="ping",
+            method='GET',
+            expected_response_code=204
+        )
+
+        return response.headers['X-Influxdb-Version']
+
+    @staticmethod
+    def _batches(iterable, size):
         for i in xrange(0, len(iterable), size):
             yield iterable[i:i + size]
 
@@ -471,11 +519,6 @@ localhost:8086/databasename', timeout=5, udp_port=159)
                 "Invalid time precision is given. "
                 "(use 'n', 'u', 'ms', 's', 'm' or 'h')")
 
-        if self.use_udp and time_precision and time_precision != 's':
-            raise ValueError(
-                "InfluxDB only supports seconds precision for udp writes"
-            )
-
         if protocol == 'json':
             data = {
                 'points': points
@@ -496,8 +539,10 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         if retention_policy is not None:
             params['rp'] = retention_policy
 
-        if self.use_udp:
-            self.send_packet(data, protocol=protocol)
+        if self._use_udp:
+            self.send_packet(
+                data, protocol=protocol, time_precision=time_precision
+            )
         else:
             self.write(
                 data=data,
@@ -530,7 +575,8 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :param dbname: the name of the database to create
         :type dbname: str
         """
-        self.query("CREATE DATABASE {0}".format(quote_ident(dbname)))
+        self.query("CREATE DATABASE {0}".format(quote_ident(dbname)),
+                   method="POST")
 
     def drop_database(self, dbname):
         """Drop a database from InfluxDB.
@@ -538,10 +584,39 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :param dbname: the name of the database to drop
         :type dbname: str
         """
-        self.query("DROP DATABASE {0}".format(quote_ident(dbname)))
+        self.query("DROP DATABASE {0}".format(quote_ident(dbname)),
+                   method="POST")
+
+    def get_list_measurements(self):
+        """Get the list of measurements in InfluxDB.
+
+        :returns: all measurements in InfluxDB
+        :rtype: list of dictionaries
+
+        :Example:
+
+        ::
+
+            >> dbs = client.get_list_measurements()
+            >> dbs
+            [{u'name': u'measurements1'},
+             {u'name': u'measurements2'},
+             {u'name': u'measurements3'}]
+        """
+        return list(self.query("SHOW MEASUREMENTS").get_points())
+
+    def drop_measurement(self, measurement):
+        """Drop a measurement from InfluxDB.
+
+        :param measurement: the name of the measurement to drop
+        :type measurement: str
+        """
+        self.query("DROP MEASUREMENT {0}".format(quote_ident(measurement)),
+                   method="POST")
 
     def create_retention_policy(self, name, duration, replication,
-                                database=None, default=False):
+                                database=None,
+                                default=False, shard_duration="0s"):
         """Create a retention policy for a database.
 
         :param name: the name of the new retention policy
@@ -549,8 +624,8 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :param duration: the duration of the new retention policy.
             Durations such as 1h, 90m, 12h, 7d, and 4w, are all supported
             and mean 1 hour, 90 minutes, 12 hours, 7 day, and 4 weeks,
-            respectively. For infinite retention – meaning the data will
-            never be deleted – use 'INF' for duration.
+            respectively. For infinite retention - meaning the data will
+            never be deleted - use 'INF' for duration.
             The minimum retention period is 1 hour.
         :type duration: str
         :param replication: the replication of the retention policy
@@ -560,21 +635,31 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :type database: str
         :param default: whether or not to set the policy as default
         :type default: bool
+        :param shard_duration: the shard duration of the retention policy.
+            Durations such as 1h, 90m, 12h, 7d, and 4w, are all supported and
+            mean 1 hour, 90 minutes, 12 hours, 7 day, and 4 weeks,
+            respectively. Infinite retention is not supported. As a workaround,
+            specify a "1000w" duration to achieve an extremely long shard group
+            duration. Defaults to "0s", which is interpreted by the database
+            to mean the default value given the duration.
+            The minimum shard group duration is 1 hour.
+        :type shard_duration: str
         """
         query_string = \
             "CREATE RETENTION POLICY {0} ON {1} " \
-            "DURATION {2} REPLICATION {3}".format(
+            "DURATION {2} REPLICATION {3} SHARD DURATION {4}".format(
                 quote_ident(name), quote_ident(database or self._database),
-                duration, replication)
+                duration, replication, shard_duration)
 
         if default is True:
             query_string += " DEFAULT"
 
-        self.query(query_string)
+        self.query(query_string, method="POST")
 
     def alter_retention_policy(self, name, database=None,
-                               duration=None, replication=None, default=None):
-        """Mofidy an existing retention policy for a database.
+                               duration=None, replication=None,
+                               default=None, shard_duration=None):
+        """Modify an existing retention policy for a database.
 
         :param name: the name of the retention policy to modify
         :type name: str
@@ -584,8 +669,8 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :param duration: the new duration of the existing retention policy.
             Durations such as 1h, 90m, 12h, 7d, and 4w, are all supported
             and mean 1 hour, 90 minutes, 12 hours, 7 day, and 4 weeks,
-            respectively. For infinite retention – meaning the data will
-            never be deleted – use 'INF' for duration.
+            respectively. For infinite retention, meaning the data will
+            never be deleted, use 'INF' for duration.
             The minimum retention period is 1 hour.
         :type duration: str
         :param replication: the new replication of the existing
@@ -593,21 +678,32 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :type replication: int
         :param default: whether or not to set the modified policy as default
         :type default: bool
+        :param shard_duration: the shard duration of the retention policy.
+            Durations such as 1h, 90m, 12h, 7d, and 4w, are all supported and
+            mean 1 hour, 90 minutes, 12 hours, 7 day, and 4 weeks,
+            respectively. Infinite retention is not supported. As a workaround,
+            specify a "1000w" duration to achieve an extremely long shard group
+            duration.
+            The minimum shard group duration is 1 hour.
+        :type shard_duration: str
 
         .. note:: at least one of duration, replication, or default flag
             should be set. Otherwise the operation will fail.
         """
         query_string = (
             "ALTER RETENTION POLICY {0} ON {1}"
-        ).format(quote_ident(name), quote_ident(database or self._database))
+        ).format(quote_ident(name),
+                 quote_ident(database or self._database), shard_duration)
         if duration:
             query_string += " DURATION {0}".format(duration)
+        if shard_duration:
+            query_string += " SHARD DURATION {0}".format(shard_duration)
         if replication:
             query_string += " REPLICATION {0}".format(replication)
         if default is True:
             query_string += " DEFAULT"
 
-        self.query(query_string)
+        self.query(query_string, method="POST")
 
     def drop_retention_policy(self, name, database=None):
         """Drop an existing retention policy for a database.
@@ -621,7 +717,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         query_string = (
             "DROP RETENTION POLICY {0} ON {1}"
         ).format(quote_ident(name), quote_ident(database or self._database))
-        self.query(query_string)
+        self.query(query_string, method="POST")
 
     def get_list_retention_policies(self, database=None):
         """Get the list of retention policies for a database.
@@ -642,8 +738,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
               u'duration': u'0',
               u'name': u'default',
               u'replicaN': 1}]
-            """
-
+        """
         if not (database or self._database):
             raise InfluxDBClientError(
                 "get_list_retention_policies() requires a database as a "
@@ -674,7 +769,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         return list(self.query("SHOW USERS").get_points())
 
     def create_user(self, username, password, admin=False):
-        """Create a new user in InfluxDB
+        """Create a new user in InfluxDB.
 
         :param username: the new username to create
         :type username: str
@@ -688,7 +783,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
             quote_ident(username), quote_literal(password))
         if admin:
             text += ' WITH ALL PRIVILEGES'
-        self.query(text)
+        self.query(text, method="POST")
 
     def drop_user(self, username):
         """Drop a user from InfluxDB.
@@ -696,8 +791,8 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         :param username: the username to drop
         :type username: str
         """
-        text = "DROP USER {0}".format(quote_ident(username))
-        self.query(text)
+        text = "DROP USER {0}".format(quote_ident(username), method="POST")
+        self.query(text, method="POST")
 
     def set_user_password(self, username, password):
         """Change the password of an existing user.
@@ -712,8 +807,9 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         self.query(text)
 
     def delete_series(self, database=None, measurement=None, tags=None):
-        """Delete series from a database. Series can be filtered by
-        measurement and tags.
+        """Delete series from a database.
+
+        Series can be filtered by measurement and tags.
 
         :param database: the database from which the series should be
             deleted, defaults to client's current database
@@ -732,7 +828,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
             tag_eq_list = ["{0}={1}".format(quote_ident(k), quote_literal(v))
                            for k, v in tags.items()]
             query_str += ' WHERE ' + ' AND '.join(tag_eq_list)
-        self.query(query_str, database=database)
+        self.query(query_str, database=database, method="POST")
 
     def grant_admin_privileges(self, username):
         """Grant cluster administration privileges to a user.
@@ -744,7 +840,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
             and manage users.
         """
         text = "GRANT ALL PRIVILEGES TO {0}".format(quote_ident(username))
-        self.query(text)
+        self.query(text, method="POST")
 
     def revoke_admin_privileges(self, username):
         """Revoke cluster administration privileges from a user.
@@ -756,7 +852,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
             and manage users.
         """
         text = "REVOKE ALL PRIVILEGES FROM {0}".format(quote_ident(username))
-        self.query(text)
+        self.query(text, method="POST")
 
     def grant_privilege(self, privilege, database, username):
         """Grant a privilege on a database to a user.
@@ -772,7 +868,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         text = "GRANT {0} ON {1} TO {2}".format(privilege,
                                                 quote_ident(database),
                                                 quote_ident(username))
-        self.query(text)
+        self.query(text, method="POST")
 
     def revoke_privilege(self, privilege, database, username):
         """Revoke a privilege on a database from a user.
@@ -788,7 +884,7 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         text = "REVOKE {0} ON {1} FROM {2}".format(privilege,
                                                    quote_ident(database),
                                                    quote_ident(username))
-        self.query(text)
+        self.query(text, method="POST")
 
     def get_list_privileges(self, username):
         """Get the list of all privileges granted to given user.
@@ -812,23 +908,35 @@ localhost:8086/databasename', timeout=5, udp_port=159)
         text = "SHOW GRANTS FOR {0}".format(quote_ident(username))
         return list(self.query(text).get_points())
 
-    def send_packet(self, packet, protocol='json'):
+    def send_packet(self, packet, protocol='json', time_precision=None):
         """Send an UDP packet.
 
         :param packet: the packet to be sent
         :type packet: (if protocol is 'json') dict
-                      (if protocol is 'line') sequence of line protocol strings
+                      (if protocol is 'line') list of line protocol strings
         :param protocol: protocol of input data, either 'json' or 'line'
         :type protocol: str
+        :param time_precision: Either 's', 'm', 'ms' or 'u', defaults to None
+        :type time_precision: str
         """
         if protocol == 'json':
-            data = make_lines(packet).encode('utf-8')
+            data = make_lines(packet, time_precision).encode('utf-8')
         elif protocol == 'line':
             data = ('\n'.join(packet) + '\n').encode('utf-8')
-        self.udp_socket.sendto(data, (self._host, self.udp_port))
+        self.udp_socket.sendto(data, (self._host, self._udp_port))
+
+    def close(self):
+        """Close http session."""
+        if isinstance(self._session, requests.Session):
+            self._session.close()
 
 
-def parse_dsn(dsn):
+def _parse_dsn(dsn):
+    """Parse data source name.
+
+    This is a helper function to split the data source name provided in
+    the from_dsn classmethod
+    """
     conn_params = urlparse(dsn)
     init_args = {}
     scheme_info = conn_params.scheme.split('+')
